;##################################################################
; MP3INFO
;##################################################################

#include "MP3Info.h"

;	Version 2.0 frames

#define TAGV2_BUF	"BUF"	;Recommended buffer size
#define TAGV2_CNT	"CNT"	;Play counter
#define TAGV2_COM	"COM"	;Comments
#define TAGV2_CRA	"CRA"	;Audio encryption
#define TAGV2_CRM	"CRM"	;Encrypted meta frame
#define TAGV2_ETC	"ETC"	;Event timing codes
#define TAGV2_EQU	"EQU"	;Equalization
#define TAGV2_GEO	"GEO"	;General encapsulated object
#define TAGV2_IPL	"IPL"	;Involved people list
#define TAGV2_LNK	"LNK"	;Linked information
#define TAGV2_MCI	"MCI"	;Music CD Identifier
#define TAGV2_MLL	"MLL"	;MPEG location lookup table
#define TAGV2_PIC	"PIC"	;Attached picture
#define TAGV2_POP	"POP"	;Popularimeter
#define TAGV2_REV	"REV"	;Reverb
#define TAGV2_RVA	"RVA"	;Relative volume adjustment
#define TAGV2_SLT	"SLT"	;Synchronized lyric/text
#define TAGV2_STC	"STC"	;Synced tempo codes
#define TAGV2_TAL	"TAL"	;Album/Movie/Show title
#define TAGV2_TBP	"TBP"	;BPM (Beats Per Minute)
#define TAGV2_TCM	"TCM"	;Composer
#define TAGV2_TCO	"TCO"	;Content type
#define TAGV2_TCR	"TCR"	;Copyright message
#define TAGV2_TDA	"TDA"	;Date
#define TAGV2_TDY	"TDY"	;Playlist delay
#define TAGV2_TEN	"TEN"	;Encoded by
#define TAGV2_TFT	"TFT"	;File type
#define TAGV2_TIM	"TIM"	;Time
#define TAGV2_TKE	"TKE"	;Initial key
#define TAGV2_TLA	"TLA"	;Language(s)
#define TAGV2_TLE	"TLE"	;Length
#define TAGV2_TMT	"TMT"	;Media type
#define TAGV2_TOA	"TOA"	;Original artist(s)/performer(s)
#define TAGV2_TOF	"TOF"	;Original filename
#define TAGV2_TOL	"TOL"	;Original Lyricist(s)/text writer(s)
#define TAGV2_TOR	"TOR"	;Original release year
#define TAGV2_TOT	"TOT"	;Original album/Movie/Show title
#define TAGV2_TP1	"TP1"	;Lead artist(s)/Lead performer(s)/Soloist(s)/Performing group
#define TAGV2_TP2	"TP2"	;Band/Orchestra/Accompaniment
#define TAGV2_TP3	"TP3"	;Conductor/Performer refinement
#define TAGV2_TP4	"TP4"	;Interpreted, remixed, or otherwise modified by
#define TAGV2_TPA	"TPA"	;Part of a set
#define TAGV2_TPB	"TPB"	;Publisher
#define TAGV2_TRC	"TRC"	;ISRC (International Standard Recording Code)
#define TAGV2_TRD	"TRD"	;Recording dates
#define TAGV2_TRK	"TRK"	;Track number/Position in set
#define TAGV2_TSI	"TSI"	;Size
#define TAGV2_TSS	"TSS"	;Software/hardware and settings used for encoding
#define TAGV2_TT1	"TT1"	;Content group description
#define TAGV2_TT2	"TT2"	;Title/Songname/Content description
#define TAGV2_TT3	"TT3"	;Subtitle/Description refinement
#define TAGV2_TXT	"TXT"	;Lyricist/text writer
;#define TAGV2_TXX	"TXX"	;User defined text information frame
#define TAGV2_TYE	"TYE"	;Year
#define TAGV2_UFI	"UFI"	;Unique file identifier
#define TAGV2_ULT	"ULT"	;Unsychronized lyric/text transcription
#define TAGV2_WAF	"WAF"	;Official audio file webpage
#define TAGV2_WAR	"WAR"	;Official artist/performer webpage
#define TAGV2_WAS	"WAS"	;Official audio source webpage
#define TAGV2_WCM	"WCM"	;Commercial information
#define TAGV2_WCP	"WCP"	;Copyright/Legal information
#define TAGV2_WPB	"WPB"	;Publishers official webpage
;#define TAGV2_WXX	"WXX"	;User defined URL link frame

;	Version 3.0/4.0 frames

#define TAGV3_AENC	"AENC" ;Audio encryption
#define TAGV3_APIC	"APIC" ;Attached picture
#define TAGV3_ASPI	"ASPI" ;Audio seek point index (V4)
#define TAGV3_COMM	"COMM" ;Comments
#define TAGV3_COMR	"COMR" ;Commercial frame
#define TAGV3_ENCR	"ENCR" ;Encryption method registration
#define TAGV3_EQU2	"EQU2" ;Equalisation (2) (V4)
#define TAGV3_EQUA	"EQUA" ;Equalisation (V3)
#define TAGV3_ETCO	"ETCO" ;Event timing codes
#define TAGV3_GEOB	"GEOB" ;General encapsulated object
#define TAGV3_GRID	"GRID" ;Group identification registration
#define TAGV3_IPLS	"IPLS" ;Involved people list (V3)
#define TAGV3_LINK	"LINK" ;Linked information
#define TAGV3_MCDI	"MCDI" ;Music CD identifier
#define TAGV3_MLLT	"MLLT" ;MPEG location lookup table
#define TAGV3_OWNE	"OWNE" ;Ownership frame
#define TAGV3_PRIV	"PRIV" ;Private frame
#define TAGV3_PCNT	"PCNT" ;Play counter
#define TAGV3_POPM	"POPM" ;Popularimeter
#define TAGV3_POSS	"POSS" ;Position synchronisation frame
#define TAGV3_RBUF	"RBUF" ;Recommended buffer size
#define TAGV3_RVAD	"RVAD" ;Relative volume adjustment (V3)
#define TAGV3_RVA2	"RVA2" ;Relative volume adjustment (2) (V4)
#define TAGV3_RVRB	"RVRB" ;Reverb
#define TAGV3_SEEK	"SEEK" ;Seek frame (V4)
#define TAGV3_SIGN	"SIGN" ;Signature frame (V4)
#define TAGV3_SYLT	"SYLT" ;Synchronised lyric/text
#define TAGV3_SYTC	"SYTC" ;Synchronised tempo codes
#define TAGV3_TALB	"TALB" ;Album/Movie/Show title
#define TAGV3_TBPM	"TBPM" ;BPM (beats per minute)
#define TAGV3_TCOM	"TCOM" ;Composer
#define TAGV3_TCON	"TCON" ;Content type
#define TAGV3_TCOP	"TCOP" ;Copyright message
#define TAGV3_TDAT	"TDAT" ;Date (V3)
#define TAGV3_TDEN	"TDEN" ;Encoding time
#define TAGV3_TDRC	"TDRC" ;Recording time (V4)
#define TAGV3_TDLY	"TDLY" ;Playlist delay
#define TAGV3_TDOR	"TDOR" ;Original release time (V4)
#define TAGV3_TDRL	"TDRL" ;Release time (V4)
#define TAGV3_TDTG	"TDTG" ;Tagging time (V4)
#define TAGV3_TENC	"TENC" ;Encoded by
#define TAGV3_TEXT	"TEXT" ;Lyricist/Text writer
#define TAGV3_TFLT	"TFLT" ;File type
#define TAGV3_TIPL	"TIPL" ;Involved people list (V4)
#define TAGV3_TIME	"TIME" ;Time (V3)
#define TAGV3_TIT1	"TIT1" ;Content group description
#define TAGV3_TIT2	"TIT2" ;Title/songname/content description
#define TAGV3_TIT3	"TIT3" ;Subtitle/Description refinement
#define TAGV3_TKEY	"TKEY" ;Initial key
#define TAGV3_TLAN	"TLAN" ;Language(s)
#define TAGV3_TLEN	"TLEN" ;Length
#define TAGV3_TMCL	"TMCL" ;Musician credits list (V4)
#define TAGV3_TMED	"TMED" ;Media type
#define TAGV3_TMOO	"TMOO" ;Mood (V4)
#define TAGV3_TOAL	"TOAL" ;Original album/movie/show title
#define TAGV3_TOFN	"TOFN" ;Original filename
#define TAGV3_TOLY	"TOLY" ;Original lyricist(s)/text writer(s)
#define TAGV3_TOPE	"TOPE" ;Original artist(s)/performer(s)
#define TAGV3_TORY	"TORY" ;Original release year (V3)
#define TAGV3_TOWN	"TOWN" ;File owner/licensee
#define TAGV3_TPE1	"TPE1" ;Lead performer(s)/Soloist(s)
#define TAGV3_TPE2	"TPE2" ;Band/orchestra/accompaniment
#define TAGV3_TPE3	"TPE3" ;Conductor/performer refinement
#define TAGV3_TPE4	"TPE4" ;Interpreted, remixed, or otherwise modified by
#define TAGV3_TPOS	"TPOS" ;Part of a set
#define TAGV3_TPRO	"TPRO" ;Produced notice (V4)
#define TAGV3_TPUB	"TPUB" ;Publisher
#define TAGV3_TRCK	"TRCK" ;Track number/Position in set
#define TAGV3_TRDA	"TRDA" ;Recording dates (V3)
#define TAGV3_TRSN	"TRSN" ;Internet radio station name
#define TAGV3_TRSO	"TRSO" ;Internet radio station owner
#define TAGV3_TSIZ	"TSIZ" ;Size (V3)
#define TAGV3_TSOA	"TSOA" ;Album sort order (V4)
#define TAGV3_TSOP	"TSOP" ;Performer sort order (V4)
#define TAGV3_TSOT	"TSOT" ;Title sort order (V4)
#define TAGV3_TSRC	"TSRC" ;ISRC (international standard recording code)
#define TAGV3_TSSE	"TSSE" ;Software/Hardware and settings used for encoding
#define TAGV3_TSST	"TSST" ;Set subtitle (V4)
#define TAGV3_TYER	"TYER" ;Year (V3)
;#define TAGV3_TXXX	"TXXX" ;User defined text information frame (TXXX is a placeholder do not use)
#define TAGV3_UFID	"UFID" ;Unique file identifier
#define TAGV3_USER	"USER" ;Terms of use
#define TAGV3_USLT	"USLT" ;Unsynchronised lyric/text transcription
#define TAGV3_WCOM	"WCOM" ;Commercial information
#define TAGV3_WCOP	"WCOP" ;Copyright/Legal information
#define TAGV3_WOAF	"WOAF" ;Official audio file webpage
#define TAGV3_WOAR	"WOAR" ;Official artist/performer webpage
#define TAGV3_WOAS	"WOAS" ;Official audio source webpage
#define TAGV3_WORS	"WORS" ;Official Internet radio station homepage
#define TAGV3_WPAY	"WPAY" ;Payment
#define TAGV3_WPUB	"WPUB" ;Publishers official webpage
;#define TAGV3_WXXX	"WXXX" ;User defined URL link frame (WXXX is a placeholder do not use)

#define id3v4	04334449h
#define id3v3	03334449h
#define id3v2	02334449h
#define Xing	1735289176

DATA SECTION
	align 4
	; Sample rate translation matrix
	V1L1						DD			0,32,64,96,128,160,192,224,256,288,320,352,384,416,448
	V1L2						DD			0,32,48,56,64,80,96,112,128,160,192,224,256,320,384
	V1L3						DD			0,32,40,48,56,64,80,96,112,128,160,192,224,256,320
	V2L1						DD			0,32,48,56,64,80,96,112,128,144,160,176,192,224,256
	V2L2						DD			0,8,16,24,32,40,48,56,64,80,96,112,128,144,160
	V2L3						DD			0,8,16,24,32,40,48,56,64,80,96,112,128,144,160
	align 4
	CompressedFrame				DB			"Frame compression used",0
	align 4
	EncryptedFrame				DB			"Frame encryption used",0

	align 4
	TagBuffer					DD			256	DUP	(?)
	NoData						DD			?		; Flag to indicate a COMM field
	FrameType					DD			?		; Holder for frame type
	Dummy						DD			?
	flSIZE						DD			?		; File size in bytes
	ahBITR						DD			?		; Bitrate of audio
	ahSMPR						DD			?		; Samplerate of audio
	ahVBRE						DD			?		; VBR encoded (0 = no, 1 = yes)
	ahLNTH						DD			?		; Length of track in milliseconds
	ahMPEG						DD			?		; MPEG encoding type
	ahLAYR						DD			?		; MPEG layer
	ahSTRT						DD			?		; Start of audio header
	ahSIZE						DD			?		; Size of audio frame in bytes
	ahPRIV						DD			?		; Private flag
	ahCHAN						DD			?		; Channel flag
	ahPROT						DD			?		; Protection flag
	ahPADD						DD			?		; Padding flag
	ahNFRM						DD			?		; Number of frames
	ahLFRM						DD			?		; Length of frame in milliseconds
	ahCOPR						DD			?		; Copyright Bit
	ahORIG						DD			?		; Original Bit
	ahEMPH						DD			?		; Emphasis
	ahJSMX						DD			?		; Joint stereo mode extension
	hFile						DD			?		; Handle of the MP3 file
	hMapFile					DD			?		; Handle of the file map object
	MapFileOffset				DD			?		; Offset to byte 0 of file map
	FileSize					DD			?		; Size of file in bytes

	hInstance			DD 		?
	hNoImage			DD		?

	szMP3OpenCaption	DB	"open MP3 file",0
	szMP3DefExt			DB	"mp3",0
	szMP3FilterStr		DB	"MP3 audio files (*.mp3)",0,"*.mp3",0,"All files (*.*)",0,"*.*",0,0

	ainfo				AUDIOINFO <?>
	id3info				ID3TAG <?>
	FileName			DB	MAX_PATH DUP (?)
	gdiptoken			DD	?
	gdipsi				GdiplusStartupInput <1,0,0,0>

CODE SECTION

Start:
	invoke GetModuleHandle, 0
	mov [hInstance],eax
	// Start COM
	invoke CoInitialize,NULL
	// Start GDI+
	invoke GdiplusStartup,offset gdiptoken,offset gdipsi,NULL
	test eax,eax
	jnz >>.ERRORGDIPLUS
	// Default image if none is found in the MP3 file
	invoke LoadImage,[hInstance],9000,IMAGE_BITMAP,0,0,NULL
	mov [hNoImage],eax
	invoke DialogBoxParam,[hInstance],IDD_DLG1,0,ADDR DlgProc,0
	invoke GdiplusShutdown,[gdiptoken]
	invoke DeleteObject,[hNoImage]
	invoke ExitProcess,0

	.ERRORGDIPLUS
	invoke MessageBox,NULL,"This program requires GDI+ support",0,0
	invoke ExitProcess,0

DlgProc FRAME hwnd,uMsg,wParam,lParam
	uses ebx,edi,esi
	LOCAL hbr	:D
	LOCAL hdc	:D
	LOCAL mdc1	:D
	LOCAL mdc2	:D
	LOCAL hbmp	:D
	LOCAL rect	:RECT
	LOCAL bmp	:BITMAP
	LOCAL pt	:POINT
	LOCAL tag[16]:B

	.WM_INITDIALOG
	cmp D[uMsg],WM_INITDIALOG
	jne >>.WM_COMMAND
		invoke GetWindowRect,[hwnd],offset rect
		mov eax,[rect.left]
		sub [rect.right],eax
		mov eax,[rect.top]
		sub [rect.bottom],eax
		mov D[rect.left],0
		mov D[rect.top],0
		invoke CreateRoundRectRgn,[rect.left],[rect.top],[rect.right],[rect.bottom],16,16
		invoke SetWindowRgn,[hwnd],eax,TRUE
		invoke GetDC,[hwnd]
		mov [hdc],eax
		invoke SetDCBrushColor,[hdc],0FFFFFFh
		invoke ReleaseDC,[hwnd],[hdc]

		jmp >>.EXIT

	.WM_COMMAND
	cmp D[uMsg],WM_COMMAND
	jne >>.WM_CLOSE
		movzx eax,W[wParam+2]
		movzx edx,W[wParam]
		cmp edx,1008
		jne >>.EXIT

		invoke OpenMP3File,offset FileName
		test eax,eax
		jz >>.EXIT

		invoke GetAudioInfo,offset FileName,offset ainfo
		invoke SetDlgItemInt,[hwnd],IDC_BTR,[ainfo.Bitrate],FALSE
		invoke SetDlgItemInt,[hwnd],IDC_SMPRT,[ainfo.Samplerate],FALSE
		invoke SetDlgItemInt,[hwnd],IDC_NFRMS,[ainfo.nFrames],FALSE
		invoke SetDlgItemInt,[hwnd],IDC_LENGTH,[ainfo.Length],FALSE
		invoke SetDlgItemInt,[hwnd],IDC_OFFSET,[ainfo.pStart],FALSE
		
		cmp D[ainfo.MPEGType],3
		jne >
		invoke SetDlgItemText,[hwnd],IDC_TYPE,"MPEG V1.0"
		jmp >.1
		:
		cmp D[ainfo.MPEGType],2
		jne >
		invoke SetDlgItemText,[hwnd],IDC_TYPE,"MPEG V2.0"
		jmp >.1
		:
		invoke SetDlgItemText,[hwnd],IDC_TYPE,"MPEG V2.5"
		.1

		invoke SetDlgItemInt,[hwnd],IDC_LAYER,[ainfo.Layer],FALSE
		

		invoke SetDlgItemText,[hwnd],IDC_CPYRT,"FALSE"
		cmp D[ainfo.Copyright],1
		jne >
		invoke SetDlgItemText,[hwnd],IDC_CPYRT,"TRUE"
		:
		
		invoke SetDlgItemText,[hwnd],IDC_VBR,"FALSE"
		cmp D[ainfo.IsVBR],1
		jne >
		invoke SetDlgItemText,[hwnd],IDC_VBR,"TRUE"
		:

		cmp D[ainfo.Channels],0
		jne >
			invoke SetDlgItemText,[hwnd],IDC_CHAN,"Stereo"
			jmp >.2
		:
		cmp D[ainfo.Channels],1
		jne >
			invoke SetDlgItemText,[hwnd],IDC_CHAN,"Joint Stereo"
			jmp >.2
		:
		cmp D[ainfo.Channels],2
		jne >
			invoke SetDlgItemText,[hwnd],IDC_CHAN,"Dual Channel"
			jmp >.2
		:
			invoke SetDlgItemText,[hwnd],IDC_CHAN,"Mono"
		.2

		invoke lstrcpy,offset tag,"No tag"
		invoke ReadIDtag,offset FileName,offset id3info
		test eax,eax
		jz >
		invoke wsprintf,offset tag,"ID3V%u",eax
		add esp,12
		:
		invoke SetDlgItemText,[hwnd],IDC_TAG,offset tag
		
		invoke SetDlgItemText,[hwnd],IDC_TITLE,offset id3info.TIT2
		invoke SetDlgItemText,[hwnd],IDC_ARTIST,offset id3info.TPE1
		invoke SetDlgItemText,[hwnd],IDC_ALBUM,offset id3info.TALB
		invoke SetDlgItemText,[hwnd],IDC_YEAR,offset id3info.TDRC
		
		// Create a number from TCON
		mov eax,offset id3info.TCON
		mov cl,[eax]
		cmp cl,"("
		jne >
		inc eax
		invoke a2dw,eax
		add eax,2000
		mov ebx,eax
		invoke LoadString,[hInstance],ebx,offset id3info.TCON,255
		:
		invoke SetDlgItemText,[hwnd],IDC_GENRE,offset id3info.TCON

		invoke SetDlgItemText,[hwnd],IDC_COMMENT,offset id3info.TCOM

		cmp D[id3info.APIC],0
		jne >
		mov eax,[hNoImage]
		mov [id3info.APIC],eax
		:
		invoke GetObject,[id3info.APIC],SIZEOF BITMAP,offset bmp

		// Resize the image to fit the picturebox
		invoke GetDlgItem,[hwnd],IDC_COVERART
		invoke GetClientRect,eax,offset rect
		invoke GetDC,[hwnd]
		mov [hdc],eax
		invoke CreateCompatibleDC,[hdc]
		mov [mdc1],eax
		invoke CreateCompatibleDC,[hdc]
		mov [mdc2],eax
		invoke CreateCompatibleBitmap,[hdc],[rect.right],[rect.bottom]
		mov [hbmp],eax
		invoke ReleaseDC,[hwnd],[hdc]

		// StretchBlt draws lousy pics by default, using HALFTONE makes a big difference
		invoke SetBrushOrgEx,[mdc1],0,0,offset pt
		invoke SetStretchBltMode,[mdc1],HALFTONE

		invoke SelectObject,[mdc1],[hbmp]
		mov [hbmp],eax
		invoke SelectObject,[mdc2],[id3info.APIC]
		mov [id3info.APIC],eax

		invoke StretchBlt,[mdc1],0,0,[rect.right],[rect.bottom],[mdc2],0,0,[bmp.bmWidth],[bmp.bmHeight],SRCCOPY

		invoke SelectObject,[mdc1],[hbmp]
		mov [hbmp],eax
		invoke SelectObject,[mdc2],[id3info.APIC]
		mov [id3info.APIC],eax

		invoke DeleteDC,[mdc1]
		invoke DeleteDC,[mdc2]
		invoke DeleteObject,[id3info.APIC]

		invoke SendDlgItemMessage,[hwnd],IDC_COVERART,STM_SETIMAGE,IMAGE_BITMAP,[hbmp]
		test eax,eax
		jz >
			invoke DeleteObject,eax
		:

		jmp >>.EXIT

	.WM_CLOSE
	cmp D[uMsg],WM_CLOSE
	jne >>.DEFPROC
		// Get whatever image is in the coverart box and delete it
		invoke SendDlgItemMessage,[hwnd],IDC_COVERART,STM_GETIMAGE,IMAGE_BITMAP,0
		test eax,eax
		jz >
			invoke DeleteObject,eax
		:
		INVOKE EndDialog,[hwnd],0

	.DEFPROC
		mov eax,FALSE
		ret

	.EXIT

	mov eax, TRUE
	ret
ENDF

OpenMP3File FRAME pszFileName
	LOCAL ofn	:OPENFILENAME

	mov D[ofn.lStructSize],SIZEOF OPENFILENAME
	mov D[ofn.hwndOwner],NULL
	mov D[ofn.hInstance],NULL
	mov D[ofn.lpstrFilter],OFFSET szMP3FilterStr
	mov D[ofn.lpstrCustomFilter],NULL
	mov D[ofn.nMaxCustFilter],NULL
	mov D[ofn.nFilterIndex],1
	mov eax,[pszFileName]
	mov D[eax],0
	mov [ofn.lpstrFile],eax
	mov D[ofn.nMaxFile],MAX_PATH
	mov D[ofn.lpstrFileTitle],NULL
	mov D[ofn.nMaxFileTitle],MAX_PATH
	mov D[ofn.lpstrInitialDir],NULL
	mov D[ofn.lpstrTitle],OFFSET szMP3OpenCaption
	mov D[ofn.Flags], OFN_FILEMUSTEXIST
	mov D[ofn.lpfnHook],NULL
	mov D[ofn.lpTemplateName],NULL
	mov [ofn.lpstrDefExt],OFFSET szMP3DefExt
	mov D[ofn.lCustData],NULL

	invoke GetOpenFileName,OFFSET ofn
	RET
ENDF

;####################################################################
;
; UTILITY FUNCTIONS
;
;####################################################################

MemCopy FRAME lpSource,lpDest,nBytes
	uses edi,esi,ecx
	cld
	mov edi,[lpDest]
	mov esi,[lpSource]
	mov ecx,[nBytes]

	; do the evenly divisible ones
	shr ecx,2
	rep movsd

	; do the remainder
	mov ecx,[nBytes]
	and ecx,3
	rep movsb

	RET
ENDF

dw2a FRAME dwValue,lpBuffer
	uses ebx,edi

	; MichaelW w/ PDixon

		mov eax,[dwValue]
		mov ecx,2814749768
		mov edi,[lpBuffer]

		or eax,eax
		jns >S1
		neg eax
		mov B[edi],"-"
		inc edi

S1:
	push eax
;	#####fast div by 100000
		mul ecx
		mov ecx,100000
		shr edx,16
		mov eax,edx
		mov ebx,edx
		mul ecx
	sub [esp],eax

		mov eax,ebx
		mov ecx,429497
		mov ebx,10

		or eax,eax
		jz >S5

;	#######part 1
		mul ecx
		jc >D1

		mul ebx
		jc >D2

		mul ebx
		jc >D3

		mul ebx
		jc >D4

		mul ebx
		jc >D5
S5:
	pop eax
		mul ecx
		jc >D6

		mul ebx
		jc >D7

		mul ebx
		jc >D8

		mul ebx
		jc >D9

		mul ebx
		jmp >D0

;	#####end part1

D1:
		add edx,30h
		mov [edi],dl

		mul ebx
		inc edi
D2:
		add edx,30h
		mov [edi],dl

		mul ebx
		inc edi
D3:
		add edx,30h
		mov [edi],dl

		mul ebx
		inc edi
D4:
		add edx,30h
		mov [edi],dl

		mul ebx
		inc edi
D5:
		add edx,30h
		mov [edi],dl

	pop eax

		mul ecx
		inc edi
D6:
		add edx,30h
		mov [edi],dl

		mul ebx
		inc edi
D7:
		add edx,30h
		mov [edi],dl

		mul ebx
		inc edi
D8:
		add edx,30h
		mov [edi],dl

		mul ebx
		inc edi
D9:
		add edx,30h
		mov [edi],dl

		mul ebx
		inc edi
D0:
		add edx,30h
		mov [edi],dx

	mov eax,[dwValue]
	RET
ENDF

a2dw FRAME lpBuffer
	uses esi
	; The Svin

	mov esi, [lpBuffer]
	cmp B[esi],"-"
	jne  >S1
		inc esi
		xor eax,eax
		xor edx,edx
		jmp >N1
		:
		lea eax,[eax*4+eax]
		lea eax,[eax*2+edx]
		N1:
		mov dl,[esi]
		inc esi
		sub dl,"0"
		jns <
		neg eax
		ret

	S1:
		xor eax,eax
		xor edx,edx
		jmp >P1
		:
		lea eax,[eax*4+eax]
		lea eax,[eax*2+edx]
		P1:
		mov dl,[esi]
		inc esi
		sub dl,"0"
		jns <
		ret
ENDF

LoadPicture FRAME pMemory,dwSize
	LOCAL pStream	:D
	LOCAL pGlobal	:D
	LOCAL pBitmap	:D
	LOCAL hBitmap	:D

	mov D[pStream],NULL

	invoke CoTaskMemAlloc, [dwSize]
	test eax,eax
	jnz >
		; no memory available
		xor eax,eax
		ret
	:
	mov [pGlobal],eax

	invoke MemCopy,[pMemory],[pGlobal],[dwSize]

	invoke CreateStreamOnHGlobal, [pGlobal], TRUE, ADDR pStream
	test eax,eax
	jz >
		; no interface
		invoke CoTaskMemFree,[pGlobal]
		xor eax,eax
		ret
	:

	invoke GdipCreateBitmapFromStream,[pStream],offset pBitmap

	invoke GdipCreateHBITMAPFromBitmap,[pBitmap],offset hBitmap,0FFFFFFFFh

	invoke GdipDisposeImage,[pBitmap]

	CoInvoke(pStream,IStream.IUnknown.Release)

	mov eax,[hBitmap]

	RET
ENDF

;####################################################################
;
; COMMON
;
;####################################################################

openMP3	FRAME pszBUFFER
	invoke CreateFile,[pszBUFFER],GENERIC_READ,NULL,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_ARCHIVE,NULL
	mov [hFile],eax
	cmp eax, INVALID_HANDLE_VALUE
	je >.E1
	invoke CreateFileMapping,[hFile],NULL,PAGE_READONLY,0,0,NULL
	mov [hMapFile],eax
	test eax,eax
	jz >.E2
	invoke MapViewOfFile,[hMapFile],FILE_MAP_READ,0,0,0
	mov [MapFileOffset],eax
	test eax,eax
	jnz >
		invoke CloseHandle,[hMapFile]
		.E2
		invoke CloseHandle,[hFile]
		.E1
		mov eax,INVALID_HANDLE_VALUE
		mov [MapFileOffset],eax
		mov [hFile],eax
		mov [hMapFile],eax
		ret
	:

	invoke GetFileSize,[hFile],NULL
	mov [FileSize],eax
	mov eax,[hFile]
	ret
ENDF

closeMP3: USES edx
	cmp D[MapFileOffset],INVALID_HANDLE_VALUE
	je >
	invoke UnmapViewOfFile,[MapFileOffset]
	:
	cmp D[hMapFile],INVALID_HANDLE_VALUE
	je >
	invoke CloseHandle,[hMapFile]
	:
	cmp D[hFile],INVALID_HANDLE_VALUE
	je >
	invoke CloseHandle,[hFile]
	:
	ret
ENDU

DecodeSynchSafe FRAME SSVAR
	LOCAL	SSCF1				:B
	LOCAL	SSCF2				:B
	LOCAL	SSCF3				:B
	LOCAL	SSBYTE1				:B
	LOCAL	SSBYTE2				:B
	LOCAL	SSBYTE3				:B
	LOCAL	SSBYTE4				:B

	mov eax,[SSVAR]

	mov [SSBYTE4],al
	mov [SSBYTE3],ah
	bswap eax
	mov [SSBYTE1],al
	mov [SSBYTE2],ah

	mov al,[SSBYTE2]
	mov [SSCF1],al
	and B[SSCF1],1
	shl B[SSCF1],7

	mov al,[SSBYTE3]
	mov [SSCF2],al
	and B[SSCF2],3
	shl B[SSCF2],6

	mov al,[SSBYTE4]
	mov [SSCF3],al
	and B[SSCF3],7
	shl B[SSCF3],5

;	Build Bytes from data
;
	mov al,[SSCF1]
	or [SSBYTE1],al

	shr B[SSBYTE2],1
	mov al,[SSCF2]
	or [SSBYTE2],al

	shr B[SSBYTE3],2
	mov al,[SSCF3]
	or [SSBYTE3],al

	shr B[SSBYTE4],3

;	Build DWORD from bytes
;
	mov al,[SSBYTE1]
	mov ah,[SSBYTE2]
	bswap eax
	mov al,[SSBYTE4]
	mov ah,[SSBYTE3]
	bswap eax

	ret
ENDF

;####################################################################
;
; ID3 TAGS COMMON
;
;####################################################################

GetID3Type: USES edi,edx
	mov edi,[MapFileOffset]

	mov eax,[edi]

		cmp eax,id3v4
		jne >
		mov eax,4
		jmp >.EXIT

	:
		cmp eax,id3v3
		jne >
		mov eax,3
		jmp >.EXIT

	:
		cmp eax,id3v2
		jne >
		mov eax,2
		jmp >.EXIT

	:
		mov eax,0

	.EXIT

	ret
ENDU

;####################################################################
;
; V3/V4 TAGS
;
;####################################################################

GetV3TagSize: USES edi, edx
	mov edi,[MapFileOffset]
	add edi,6
	mov eax,[edi]
	invoke DecodeSynchSafe,eax
	ret
ENDU

GetV3FrameType FRAME FrameStart
	uses edi, edx
	mov edx,[FrameStart]
	mov edi,[MapFileOffset]
	mov eax,[edi+edx]
	ret
ENDF

GetNextV3FramePos FRAME FrameStart, TagType
	uses edi, edx 
	mov edi,[MapFileOffset]
	mov edx,[FrameStart]
	mov eax,[edi+edx+4]

	cmp D[TagType],4
	jl >
	// V4
	invoke DecodeSynchSafe,eax
	bswap eax
	:
	bswap eax

	cmp eax,1
	jne >
	cmp D[FrameType],TAGV3_COMM
	je >
		mov D[NoData],1
		jmp >.EXIT
	:
	cmp eax,3
	jne >
	cmp D[FrameType],TAGV3_COMM
	je >
		mov D[NoData],1
		jmp >.EXIT
	:
	mov D[NoData],0

	.EXIT
	add eax,10
	add eax,[FrameStart]
	ret
ENDF

GetV3FrameCharType FRAME FrameStart
	uses edi, edx
	xor eax,eax
	mov edi,[MapFileOffset]
	mov edx,[FrameStart]
	mov al,[edi+edx+10]
	ret
ENDF

GetV3FrameCompression FRAME FrameStart
	uses edi, edx
	xor eax,eax
	mov edi,[MapFileOffset]
	mov edx,[FrameStart]
	mov al,[edi+edx+9]
	and eax,128
	ret
ENDF

GetV3FrameEncryption FRAME FrameStart
	uses edi, edx
	xor eax,eax
	mov edi,[MapFileOffset]
	mov edx,[FrameStart]
	mov al,[edi+edx+9]
	and eax,64
	ret
ENDF

ReadV3Frame FRAME FrameStart,FrameSize,BufferAddr,NextFrame,DataType
	uses edi, edx, ecx
	LOCAL	DataIndex			:D
	LOCAL	DataBuffer[256]		:B
	LOCAL	EOFrame				:D

	cmp D[NoData],1
	jne >
		mov edi,[BufferAddr]
		mov eax,0
		mov [edi],eax
		mov eax,1
		ret
	:

	mov eax,[NextFrame]
	sub eax,1
	mov [EOFrame],eax

	mov eax,[FrameStart]
	add eax,11

	mov edi,eax
	add edi,[MapFileOffset]
	mov D[DataIndex],0
	mov edx,0
	mov ecx,0
	mov eax,0

	cmp D[DataType],1
	je >>.UNICODE

	.ANSI
		cmp eax, [EOFrame]
		jg >>.EXIT
			xor eax,eax
			mov al,[edi]
			inc edi
			test eax,eax
			jz >>.EXIT
			cmp eax,01Fh
			jle >
			cmp eax,07Fh
			jge >
			cmp D[DataIndex], 256
			jge >
				mov edx,[DataIndex]
				mov [DataBuffer+edx],al
				inc D[DataIndex]
			:
			mov eax,edi
			sub eax,[MapFileOffset]
			jmp <<.ANSI

	.UNICODE
		cmp eax, [EOFrame]
		jg >>.EXIT
			xor eax,eax
			mov ax,[edi]
			inc edi
			inc edi
			test eax,eax
			jnz >
			cmp edi,[EOFrame]
			jge >
				mov D[DataIndex],0
			:
			cmp eax,01Fh
			jle >
			cmp eax,07Fh
			jge >
			cmp D[DataIndex], 256
			jge >
				mov edx,[DataIndex]
				mov [DataBuffer+edx],al
				inc D[DataIndex]
			:
			mov eax,edi
			sub eax,[MapFileOffset]
			jmp <<.UNICODE

	.EXIT

	mov edx,[DataIndex]
	mov B[DataBuffer+edx],0
	mov eax,[DataIndex]
	inc eax
	invoke MemCopy,ADDR DataBuffer,[BufferAddr],eax

	mov eax,[DataIndex]
	ret
ENDF

;####################################################################
;
; V2 TAGS
;
;####################################################################

GetV2FrameType FRAME FrameStart
	uses edi, edx
	mov edx,[FrameStart]
	mov edi,[MapFileOffset]
	mov eax,[edi+edx]
	and eax,0FFFFFFh
	ret
ENDF

GetNextV2FramePos FRAME FrameStart, TagType
	uses edi, edx 
	mov edi,[MapFileOffset]
	mov edx,[FrameStart]
	mov eax,[edi+edx+3]
	shl eax,8
	// Only 3 bytes count
	bswap eax
	cmp eax,1
	jne >
	cmp D[FrameType],TAGV2_COM
	je >
		mov D[NoData],1
		jmp >.EXIT
	:
	cmp eax,3
	jne >
	cmp D[FrameType],TAGV2_COM
	je >
		mov D[NoData],1
		jmp >.EXIT
	:
	mov D[NoData],0

	.EXIT
	add eax,6
	add eax,[FrameStart]
	ret
ENDF

GetV2FrameCharType FRAME FrameStart
	uses edi, edx
	xor eax,eax
	mov edi,[MapFileOffset]
	mov edx,[FrameStart]
	mov al,[edi+edx+6]
	ret
ENDF

ReadV2Frame FRAME FrameStart,FrameSize,BufferAddr,NextFrame,DataType
	uses edi, edx, ecx
	LOCAL	DataIndex			:D
	LOCAL	DataBuffer[256]		:B
	LOCAL	EOFrame				:D

	cmp D[NoData],1
	jne >
		mov edi,[BufferAddr]
		mov eax,0
		mov [edi],eax
		mov eax,1
		ret
	:

	mov eax,[NextFrame]
	sub eax,1
	mov [EOFrame],eax

	mov eax,[FrameStart]
	add eax,7

	mov edi,eax
	add edi,[MapFileOffset]
	mov D[DataIndex],0
	mov edx,0
	mov ecx,0
	mov eax,0

	cmp D[DataType],1
	je >>.UNICODE

	.ANSI
		cmp eax, [EOFrame]
		jg >>.EXIT
			xor eax,eax
			mov al,[edi]
			inc edi
			test eax,eax
			jz >>.EXIT
			cmp eax,01Fh
			jle >
			cmp eax,07Fh
			jge >
			cmp D[DataIndex], 256
			jge >
				mov edx,[DataIndex]
				mov [DataBuffer+edx],al
				inc D[DataIndex]
			:
			mov eax,edi
			sub eax,[MapFileOffset]
			jmp <<.ANSI

	.UNICODE
		cmp eax, [EOFrame]
		jg >>.EXIT
			xor eax,eax
			mov ax,[edi]
			inc edi
			inc edi
			test eax,eax
			jnz >
			cmp edi,[EOFrame]
			jge >
				mov D[DataIndex],0
			:
			cmp eax,01Fh
			jle >
			cmp eax,07Fh
			jge >
			cmp D[DataIndex], 256
			jge >
				mov edx,[DataIndex]
				mov [DataBuffer+edx],al
				inc D[DataIndex]
			:
			mov eax,edi
			sub eax,[MapFileOffset]
			jmp <<.UNICODE

	.EXIT

	mov edx,[DataIndex]
	mov B[DataBuffer+edx],0
	mov eax,[DataIndex]
	inc eax
	invoke MemCopy,ADDR DataBuffer,[BufferAddr],eax

	mov eax,[DataIndex]
	ret
ENDF

;####################################################################
;
; V1 TAGS
;
;####################################################################

ReadIDV1tag FRAME lpTitle,lpAlbum,lpArtist,lpContent,lpComments,lpTrack,lpYear
	uses edi, edx, ecx
	LOCAL	DataBuffer[32]	:B

	lea edi,DataBuffer
	mov D[edi], 0
	mov D[edi + 1 * 4], 0
	mov D[edi + 2 * 4], 0

	mov edi,[lpTitle]
	mov D[edi], 0
	mov D[edi + 1 * 4], 0
	mov D[edi + 2 * 4], 0
	mov D[edi + 3 * 4], 0
	mov D[edi + 4 * 4], 0
	mov D[edi + 5 * 4], 0
	mov D[edi + 6 * 4], 0
	mov D[edi + 7 * 4], 0

	mov edi,[lpAlbum]
	mov D[edi], 0
	mov D[edi + 1 * 4], 0
	mov D[edi + 2 * 4], 0
	mov D[edi + 3 * 4], 0
	mov D[edi + 4 * 4], 0
	mov D[edi + 5 * 4], 0
	mov D[edi + 6 * 4], 0
	mov D[edi + 7 * 4], 0

	mov edi,[lpArtist]
	mov D[edi], 0
	mov D[edi + 1 * 4], 0
	mov D[edi + 2 * 4], 0
	mov D[edi + 3 * 4], 0
	mov D[edi + 4 * 4], 0
	mov D[edi + 5 * 4], 0
	mov D[edi + 6 * 4], 0
	mov D[edi + 7 * 4], 0

	mov edi,[lpYear]
	mov D[edi], 0
	mov D[edi + 1 * 4], 0

	mov edi,[lpComments]
	mov D[edi], 0
	mov D[edi + 1 * 4], 0
	mov D[edi + 2 * 4], 0
	mov D[edi + 3 * 4], 0
	mov D[edi + 4 * 4], 0
	mov D[edi + 5 * 4], 0
	mov D[edi + 6 * 4], 0
	mov D[edi + 7 * 4], 0

	mov edi,[lpTrack]
	mov D[edi], 0

	mov edi,[lpContent]
	mov D[edi], 0
	mov D[edi + 1 * 4], 0

	// Set the default content type and
	// move the pointer ahead by 1. This
	// Will make the content format match ID3V3/4
	inc D[lpContent]
	mov D[edi],"(0))"

	mov eax,[FileSize]
	cmp eax,128
	jg >
		mov eax,0
		ret
	:
	sub eax,128

	mov edi,[MapFileOffset]
	add edi,eax
	xor eax,eax

	mov eax,[edi]
	shl eax,8
	bswap eax
	cmp eax,"GAT"
	jne >>
		
		add edi,3
		invoke MemCopy,edi,[lpTitle],30
		add edi,30
		invoke MemCopy,edi,[lpArtist],30
		add edi,30
		invoke MemCopy,edi,[lpAlbum],30
		add edi,30
		invoke MemCopy,edi,[lpYear],4
		add edi,4
		invoke MemCopy,edi,[lpComments],29
		add edi,29
		xor eax,eax
		mov al,[edi]
		invoke dw2a,eax,ADDR DataBuffer
		invoke MemCopy,ADDR DataBuffer,[lpTrack],10
		add edi,1
		xor eax,eax
		mov al,[edi]
		invoke dw2a,eax,ADDR DataBuffer
		invoke MemCopy,ADDR DataBuffer,[lpContent],3
		mov eax,0
		ret
	:
		mov eax,1
		ret
ENDF

;####################################################################
;
; AUDIO
;
;####################################################################

ReadAudioData FRAME FilePointer
	uses edi, edx
	LOCAL	ahByte1			:B
	LOCAL	ahByte2			:B
	LOCAL	ahByte3			:B
	LOCAL	MaxRead			:D

	mov eax,[FilePointer]
	add eax,MAXSCANDEPTH
	cmp eax,[FileSize]
	jl >
		mov eax,[FileSize]
	:
	mov [MaxRead],eax

	mov edi,[MapFileOffset]
	add edi,[FilePointer]

	.scanfile
		inc edi
		mov eax,edi
		sub eax,[MapFileOffset]
		cmp eax,[MaxRead]
		jae >>.noheader

		xor eax,eax
		mov al,[edi]
		cmp al,255
		jne <.scanfile
		mov [ahByte1],al

		mov eax,edi
		sub eax,[MapFileOffset]
		mov [ahSTRT],eax

		inc edi
		xor eax,eax
		mov al,[edi]
		cmp al,240
		jbe <.scanfile
		mov [ahByte1],al

		xor eax,eax
		inc edi
		mov al,[edi]
		mov [ahByte2],al
		xor eax,eax
		inc edi
		mov al,[edi]
		mov [ahByte3],al
		dec edi
		dec edi
		dec edi
		cmp B[ahByte2],240
		jae <.scanfile
		jmp >>.goodsynch

	.noheader
		mov D[ahBITR],0		; Bitrate of audio
		mov D[ahSMPR],0		; Samplerate of audio
		mov D[ahVBRE],0		; VBR encoded (0 = no, 1 = yes)
		mov D[ahLNTH],0		; Length of track in milliseconds
		mov D[ahMPEG],0		; MPEG encoding type
		mov D[ahLAYR],0		; MPEG layer
		mov D[ahSTRT],0		; Start of audio header
		mov D[ahSIZE],0		; Size of audio frame in bytes
		mov D[ahPRIV],0		; Private flag
		mov D[ahCHAN],0		; Channel flag
		mov D[ahPROT],0		; Protection flag
		mov D[ahPADD],0		; Padding flag
		mov D[ahNFRM],0		; Number of frames
		mov D[ahLFRM],0		; Length of frame in milliseconds
		mov eax,-1
		ret

	.goodsynch
		sub edi,[MapFileOffset]
		invoke CheckVBR
		invoke DecodeAudioData,[ahSTRT]
		invoke DecodeRateData,[ahBITR],[ahSMPR],[ahMPEG],[ahLAYR]
		cmp D[ahVBRE],1
		jne >
			invoke DecodeVBR
		:
		mov eax,1
		ret
ENDF

DecodeAudioData FRAME BaseAddr
	uses edi, edx
	LOCAL	ahData1			:D	; Holder for audio header DWORD
	LOCAL	ahData2			:D	; Holder for audio header DWORD
	LOCAL	ahData3			:D	; Holder for audio header DWORD

	mov edi,[MapFileOffset]
	add edi,[BaseAddr]
	xor eax,eax
	mov al,[edi + 1]
	mov [ahData1],eax

	xor eax,eax
	mov al,[edi + 2]
	mov [ahData2],eax

	xor eax,eax
	mov al,[edi + 3]
	mov [ahData3],eax

	mov eax,[ahData1]
	and eax,24
	shr eax,3
	mov [ahMPEG],eax

	mov eax,[ahData1]
	and eax,6
	shr eax,1
	mov [ahLAYR],eax

	mov eax,[ahData1]
	and eax,1
	mov [ahPROT],eax

	mov eax,[ahData2]
	and eax,240
	shr eax,4
	mov [ahBITR],eax

	mov eax,[ahData2]
	and eax,12
	shr eax,2
	mov [ahSMPR],eax

	mov eax,[ahData2]
	and eax,2
	shr eax,1
	mov [ahPADD],eax

	mov eax,[ahData2]
	and eax,1
	mov [ahPRIV],eax

	mov eax,[ahData3]
	and eax,192
	shr eax,6
	mov [ahCHAN],eax

	mov eax,[ahData3]
	and eax,48
	shr eax,4
	mov [ahJSMX],eax

	mov eax,[ahData3]
	and eax,8
	shr eax,3
	mov [ahCOPR],eax

	mov eax,[ahData3]
	and eax,4
	shr eax,2
	mov [ahORIG],eax

	mov eax,[ahData3]
	and eax,3
	mov [ahEMPH],eax

	ret
ENDF

DecodeRateData FRAME DecBR,DecSR,DecMPEG,DecLAYER
	uses edi, edx, ecx
	LOCAL	BitRate	 	:D
	LOCAL	SampleRate	:D
	LOCAL	Padding		:D

	shl D[DecBR],2 ; multiply by 4 (shift left 2 is the same)

	.MPEG10
	cmp D[DecMPEG],3
	jne >>.MPEG20
		cmp D[DecLAYER],1
		jne >
			mov eax,[DecBR]
			mov eax,[V1L3+eax]
			mov [BitRate], eax
			jmp >.SMPRTV1
		:
		cmp D[DecLAYER],2
		jne >
			mov eax,[DecBR]
			mov eax,[V1L2+eax]
			mov [BitRate], eax
			jmp >.SMPRTV1
		:
		cmp D[DecLAYER],3
		jne >
			mov eax,[DecBR]
			mov eax,[V1L1+eax]
			mov [BitRate], eax
			jmp >.SMPRTV1
		:
			mov D[BitRate], 0

		.SMPRTV1
		cmp D[DecSR],0
		jne >
			mov D[SampleRate],44100
			jmp >>.ENDGETRATES
		:
		cmp D[DecSR],2
		jne >
			mov D[SampleRate],48000 ;44100
			jmp >>.ENDGETRATES
		:
		cmp D[DecSR],4
		jne >
			mov D[SampleRate],32000 ;48000
			jmp >>.ENDGETRATES
		:
		cmp D[DecSR],3
		jne >
			mov D[SampleRate],32000 ;48000
			jmp >>.ENDGETRATES
		:
	;	cmp D[DecSR],4
	;	jne >
	;		mov D[SampleRate],32000
	;		jmp >>.ENDGETRATES
		:
			mov D[SampleRate],0
			jmp >>.ENDGETRATES

	.MPEG20
	cmp D[DecMPEG],2
	jne >>.MPEG25
		cmp D[DecLAYER],1
		jne >
			mov eax,[DecBR]
			mov eax,[V2L3+eax]
			mov [BitRate], eax
			jmp >.SMPRTV2
		:
		cmp D[DecLAYER],2
		jne >
			mov eax,[DecBR]
			mov eax,[V2L2+eax]
			mov [BitRate], eax
			jmp >.SMPRTV2
		:
		cmp D[DecLAYER],3
		jne >
			mov eax,[DecBR]
			mov eax,[V2L1+eax]
			mov [BitRate], eax
			jmp >.SMPRTV2
		:
			mov D[BitRate], 0

		.SMPRTV2
		cmp D[DecSR],0
		jne >
			mov D[SampleRate],22050
			jmp >>.ENDGETRATES
		:
		cmp D[DecSR],2
		jne >
			mov D[SampleRate],24000
			jmp >>.ENDGETRATES
		:
		cmp D[DecSR],4
		jne >
			mov D[SampleRate],16000
			jmp >>.ENDGETRATES
		:
			mov D[SampleRate],0
			jmp >>.ENDGETRATES

	.MPEG25
	cmp D[DecMPEG],0
	jne >>.MPEGUNKNOWN
		cmp D[DecLAYER],1
		jne >
			mov eax,[DecBR]
			mov eax,[V2L3+eax]
			mov [BitRate], eax
			jmp >.SMPRTV25
		:
		cmp D[DecLAYER],2
		jne >
			mov eax,[DecBR]
			mov eax,[V2L2+eax]
			mov [BitRate], eax
			jmp >.SMPRTV25
		:
		cmp D[DecLAYER],3
		jne >
			mov eax,[DecBR]
			mov eax,[V2L1+eax]
			mov [BitRate], eax
			jmp >.SMPRTV25
		:
			mov D[BitRate], 0

		.SMPRTV25
		cmp D[DecSR],0
		jne >
			mov D[SampleRate],11025
			jmp >>.ENDGETRATES
		:
		cmp D[DecSR],2
		jne >
			mov D[SampleRate],12000
			jmp >>.ENDGETRATES
		:
		cmp D[DecSR],4
		jne >
			mov D[SampleRate],8000
			jmp >>.ENDGETRATES
		:
			mov D[SampleRate],0
			jmp >>.ENDGETRATES

	.MPEGUNKNOWN
		mov D[SampleRate],0
		mov D[BitRate],0

	.ENDGETRATES
	mov eax,[BitRate]
	mov ecx,1000
	mul ecx
	mov [ahBITR],eax

	mov eax,[SampleRate]
	mov [ahSMPR],eax

; Get file size
;
	mov eax,[FileSize]
	sub eax,[ahSTRT]
	sub eax,128
	mov [flSIZE],eax

	cmp D[ahLAYR],3
	jne >>
		mov eax,4
		mul D[ahPADD]
		mov [Padding],eax

		mov eax,[ahBITR]
		mov ecx,12
		mul ecx
		div D[ahSMPR]
		mov [ahSIZE],eax
		mov edx,0
		div D[ahSMPR]

		mov eax,[ahSIZE]
		shl eax,2
		add eax,[Padding]
		mov [ahSIZE],eax

		mov eax,384
		jmp >.EI

	:
		mov eax,[ahPADD]
		mov [Padding],eax

		mov eax,[ahBITR]
		mov ecx,144
		mul ecx
		div D[ahSMPR]
		add eax,[Padding]
		mov [ahSIZE],eax

		mov eax,1152
	.EI

	mov ecx,1000
	mul ecx
	div D[ahSMPR]
	mov [ahLFRM],eax

	mov eax,[flSIZE]
	mov edx,0
	div D[ahSIZE]
	mov [ahNFRM],eax

	mov eax,[ahNFRM]
	mul D[ahLFRM]
	mov [ahLNTH],eax

	ret
ENDF

CheckVBR: USES edi, edx
	mov edi,[MapFileOffset]
	add edi,[ahSTRT]
	mov eax,[edi+36]
	cmp eax,Xing
	jne >
		mov D[ahVBRE],1
		ret
	:
		mov D[ahVBRE],0
	ret
ENDU

DecodeVBR: USES edi, edx, ecx
	mov edi,[MapFileOffset]
	add edi,[ahSTRT]
	mov eax,[edi+44]
	bswap eax	; convert to big endian format
	mov [ahNFRM],eax

	mov eax,1152000
	mul D[ahNFRM]
	div D[ahSMPR]
	mov [ahLNTH],eax

	mov eax,[flSIZE]
	mov edx,0
	div D[ahNFRM]
	mov [ahLFRM],eax

	mov eax,[ahPADD]
	add [ahLFRM],eax
	mov eax,[ahSMPR]
	mul D[ahLFRM]
	mov ecx,144
	div ecx
	mov [ahBITR],eax
	ret
ENDU

;####################################################################
;
; Read procedures
;
;####################################################################

ReadIDtag FRAME pFileName,ArrayBase
	uses edi, edx, esi, ebx
	LOCAL	ID3TYPE			:D
	LOCAL	TAGSIZE			:D
	LOCAL	FRAMESIZE		:D
	LOCAL	FRAMEMAX		:D
	LOCAL	FRAMESTART		:D
	LOCAL	NEXTFRAME		:D
	LOCAL	DATALENGTH		:D
	LOCAL	FRAMECHAR		:D
	LOCAL	RETVAL			:D
	LOCAL	Encrypt			:D
	LOCAL	Compress		:D

	mov ebx,[ArrayBase]

	invoke IsBadWritePtr,ebx, SIZEOF ID3TAG
	test eax,eax
	jz >
		mov eax,-1
		ret
	:

	invoke openMP3,[pFileName]
	test eax,eax
	jnz >
	jns >
		mov eax,-1
		ret
	:

//	The main read loop

	invoke GetID3Type
	mov [ID3TYPE],eax

	cmp D[ID3TYPE],2
	je >>.IDV2TAG
	cmp D[ID3TYPE],3
	je >
	cmp D[ID3TYPE],4
	jne >>.IDV1TAG
	:
		invoke GetV3TagSize
		mov [TAGSIZE],eax
		mov D[FRAMESTART],10

		.READNEXTV3
		cmp eax,[TAGSIZE]
		jg >>.ENDREADV3
			invoke GetV3FrameType,[FRAMESTART]
			mov [FrameType],eax
			invoke GetNextV3FramePos,[FRAMESTART],[ID3TYPE]
			mov [NEXTFRAME],eax
			sub eax,10
			mov [FRAMESIZE],eax
			cmp D[NoData], 0
			jne >>.ENDCOPYV3
				invoke GetV3FrameCharType,[FRAMESTART]
				mov [FRAMECHAR],eax
				invoke GetV3FrameCompression,[FRAMESTART]
				mov [Compress],eax
				invoke GetV3FrameEncryption,[FRAMESTART]
				mov [Encrypt],eax

				cmp D[Compress], 128
				jne >
					invoke lstrcpy,ADDR TagBuffer,ADDR CompressedFrame
					jmp >.FINDTAGV3
				:
				cmp D[Encrypt], 64
				jne >
					invoke lstrcpy,ADDR TagBuffer,ADDR EncryptedFrame
					jmp >.FINDTAGV3
				:
					invoke ReadV3Frame,[FRAMESTART],[FRAMESIZE],ADDR TagBuffer,[NEXTFRAME],[FRAMECHAR]
					mov [DATALENGTH],eax

				.FINDTAGV3
				cmp D[DATALENGTH],255
				jle >
					mov D[DATALENGTH],255
				:

				cmp D[FrameType],TAGV3_TIT2
				jne >
					lea ecx,[ebx+ID3TAG.TIT2]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV3
				:
				cmp D[FrameType],TAGV3_TALB
				jne >
					lea ecx,[ebx+ID3TAG.TALB]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV3
				:
				cmp D[FrameType],TAGV3_COMM
				jne >
					lea ecx,[ebx+ID3TAG.COMM]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV3
				:
				cmp D[FrameType],TAGV3_TPE1
				jne >
					lea ecx,[ebx+ID3TAG.TPE1]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV3
				:
				cmp D[FrameType],TAGV3_TCON
				jne >
					lea ecx,[ebx+ID3TAG.TCON]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV3
				:
				cmp D[FrameType],TAGV3_TRCK
				jne >
					lea ecx,[ebx+ID3TAG.TRCK]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV3
				:
				cmp D[FrameType],TAGV3_TCOP
				jne >
					lea ecx,[ebx+ID3TAG.TCOP]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV3
				:
				cmp D[FrameType],TAGV3_TENC
				jne >
					lea ecx,[ebx+ID3TAG.TENC]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV3
				:
				cmp D[ID3TYPE],3
				je >.ID3YEAR
				cmp D[FrameType],TAGV3_TDRC
				jne >
					lea ecx,[ebx+ID3TAG.TDRC]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV3

				.ID3YEAR
				cmp D[FrameType],TAGV3_TYER
				jne >
					lea ecx,[ebx+ID3TAG.TDRC]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV3
				:

				cmp D[FrameType],TAGV3_TCOM
				jne >
					lea ecx,[ebx+ID3TAG.TCOM]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV3
				:
				cmp D[FrameType],TAGV3_TOPE
				jne >
					lea ecx,[ebx+ID3TAG.TOPE]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV3
				:
				cmp D[FrameType],TAGV3_APIC
				jne >>.ENDCOPYV3
					lea esi,[ebx+ID3TAG.APIC]
					mov D[esi],NULL
					mov eax,[FRAMESTART]
					add eax,11
					mov edi,eax
					add edi,[MapFileOffset]

					mov eax,edi
					add eax,[FRAMESIZE]
					mov [FRAMEMAX],eax

					:
					mov al,[edi]
					inc edi
					cmp edi,[FRAMEMAX]
					jae >.ENDCOPYV3
					cmp al,0
					jne <
					inc edi
					:
					mov al,[edi]
					inc edi
					cmp edi,[FRAMEMAX]
					jae >.ENDCOPYV3
					cmp al,0
					jne <
					xor eax,eax
					invoke LoadPicture,edi,[FRAMESIZE]
					mov [esi],eax
					jmp >>.ENDCOPYV3

			.ENDCOPYV3

			mov eax,[NEXTFRAME]	
			cmp eax,[FRAMESTART]
			je >
				mov [FRAMESTART],eax
				jmp >.GETNEXTFRAME
			:
				mov eax,[TAGSIZE]
				inc eax

			.GETNEXTFRAME
		jmp <<.READNEXTV3

		.ENDREADV3
		mov eax,[ID3TYPE]
		mov [RETVAL],eax
		jmp >>.DONEREAD

	.IDV2TAG
		invoke GetV3TagSize
		mov [TAGSIZE],eax
		mov D[FRAMESTART],10

		.READNEXTV2
		cmp eax,[TAGSIZE]
		jg >>.ENDREADV2
			invoke GetV2FrameType,[FRAMESTART]
			mov [FrameType],eax
			invoke GetNextV2FramePos,[FRAMESTART],[ID3TYPE]
			mov [NEXTFRAME],eax
			sub eax,6
			mov [FRAMESIZE],eax
			cmp D[NoData], 0
			jne >>.ENDCOPYV2
				invoke GetV2FrameCharType,[FRAMESTART]
				mov [FRAMECHAR],eax

				invoke ReadV2Frame,[FRAMESTART],[FRAMESIZE],ADDR TagBuffer,[NEXTFRAME],[FRAMECHAR]
				mov [DATALENGTH],eax

				.FINDTAGV2
				cmp D[DATALENGTH],255
				jle >
					mov D[DATALENGTH],255
				:

				cmp D[FrameType],TAGV2_TT2
				jne >
					lea ecx,[ebx+ID3TAG.TIT2]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV2
				:
				cmp D[FrameType],TAGV2_TAL
				jne >
					lea ecx,[ebx+ID3TAG.TALB]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV2
				:
				cmp D[FrameType],TAGV2_COM
				jne >
					lea ecx,[ebx+ID3TAG.COMM]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV2
				:
				cmp D[FrameType],TAGV2_TP1
				jne >
					lea ecx,[ebx+ID3TAG.TPE1]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV2
				:
				cmp D[FrameType],TAGV2_TCO
				jne >
					lea ecx,[ebx+ID3TAG.TCON]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV2
				:
				cmp D[FrameType],TAGV2_TRK
				jne >
					lea ecx,[ebx+ID3TAG.TRCK]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV2
				:
				cmp D[FrameType],TAGV2_TCR
				jne >
					lea ecx,[ebx+ID3TAG.TCOP]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV2
				:
				cmp D[FrameType],TAGV2_TEN
				jne >
					lea ecx,[ebx+ID3TAG.TENC]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV2
				:
				cmp D[FrameType],TAGV2_TYE
				jne >
					lea ecx,[ebx+ID3TAG.TDRC]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV2

				cmp D[FrameType],TAGV2_COM
				jne >
					lea ecx,[ebx+ID3TAG.TCOM]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV2
				:
				cmp D[FrameType],TAGV2_TOA
				jne >
					lea ecx,[ebx+ID3TAG.TOPE]
					invoke lstrcpy,ecx,ADDR TagBuffer
					jmp >>.ENDCOPYV2
				:
				cmp D[FrameType],TAGV2_PIC
				jne >>.ENDCOPYV2
					lea esi,[ebx+ID3TAG.APIC]
					mov D[esi],NULL
					mov eax,[FRAMESTART]
					add eax,12
					add eax,[MapFileOffset]
					invoke LoadPicture,eax,[FRAMESIZE]
					mov [esi],eax
					jmp >>.ENDCOPYV2

			.ENDCOPYV2

			mov eax,[NEXTFRAME]	
			cmp eax,[FRAMESTART]
			je >
				mov [FRAMESTART],eax
				jmp >.GETNEXTFRAMEV2
			:
				mov eax,[TAGSIZE]
				inc eax

			.GETNEXTFRAMEV2

		jmp <<.READNEXTV2

		.ENDREADV2
		mov eax,[ID3TYPE]
		mov [RETVAL],eax
		jmp >.DONEREAD

	.IDV1TAG
		lea ecx,[ebx+ID3TAG.TDRC]
		push ecx
		lea ecx,[ebx+ID3TAG.TRCK]
		push ecx
		lea ecx,[ebx+ID3TAG.COMM]
		push ecx
		lea ecx,[ebx+ID3TAG.TCON]
		push ecx
		lea ecx,[ebx+ID3TAG.TPE1]
		push ecx
		lea ecx,[ebx+ID3TAG.TALB]
		push ecx
		lea ecx,[ebx+ID3TAG.TIT2]
		push ecx
		invoke ReadIDV1tag
		
		cmp eax,0
		jne >
			mov D[RETVAL],1
			jmp >.DONEREAD
		:
		cmp eax,1
		jne >.DONEREAD
			mov D[RETVAL],0
			jmp >.DONEREAD
		
	.DONEREAD
;	End of main read loop

;	Close the file now that we're done
	invoke closeMP3

	mov eax, [RETVAL]
	ret
ENDF

GetAudioInfo FRAME pFileName,pAudioInfo
	uses edi, edx
	LOCAL	FoundHeader		:D

	invoke IsBadWritePtr,[pAudioInfo],SIZEOF AUDIOINFO
	test eax,eax
	jz >
		mov eax,-1
		ret
	:

	invoke openMP3,[pFileName]
	test eax,eax
	jns >
		ret
	:

	invoke GetID3Type
	cmp eax,2
	je >
	cmp eax,3
	je >
	cmp eax,4
	je >
		mov D[ahSTRT],-1
		jmp >>.READAUDIOHEADER
	:
		invoke GetV3TagSize
		mov [ahSTRT],eax

	.READAUDIOHEADER

	invoke ReadAudioData,[ahSTRT]
	mov [FoundHeader],eax

	invoke closeMP3

	mov edi,[pAudioInfo]
	mov eax,[ahBITR]
	mov [edi+AUDIOINFO.Bitrate],eax

	mov eax,[ahSMPR]
	mov [edi+AUDIOINFO.Samplerate],eax

	mov eax,[ahVBRE]
	mov [edi+AUDIOINFO.IsVBR],eax

	mov eax,[ahLNTH]
	mov [edi+AUDIOINFO.Length],eax

	mov eax,[ahMPEG]
	mov [edi+AUDIOINFO.MPEGType],eax

	cmp D[ahLAYR],1
	jne >
		mov eax,3
		jmp >.LAYER
	:
	cmp D[ahLAYR],2
	jne >
		mov eax,2
		jmp >.LAYER
	:
	mov eax,1
	.LAYER

	mov [edi+AUDIOINFO.Layer],eax

	mov eax,[ahSTRT]
	mov [edi+AUDIOINFO.pStart],eax

	mov eax,[ahORIG]
	mov [edi+AUDIOINFO.Original],eax

	mov eax,[ahPRIV]
	mov [edi+AUDIOINFO.Private],eax

	mov eax,[ahCHAN]
	mov [edi+AUDIOINFO.Channels],eax

	mov eax,[ahCOPR]
	mov [edi+AUDIOINFO.Copyright],eax

	mov eax,[ahNFRM]
	mov [edi+AUDIOINFO.nFrames],eax

	mov eax,[ahEMPH]
	mov [edi+AUDIOINFO.Emphasis],eax

	mov eax,[ahPROT]
	mov [edi+AUDIOINFO.Protected],eax

	mov eax,[ahJSMX]
	mov [edi+AUDIOINFO.JointStereoEx],eax

	mov eax,[FoundHeader]
	ret

ENDF
